@@ -39,7 +39,6 @@ G_BEGIN_DECLS
 #define NM_IS_ACTIVE_CONNECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((obj), NM_TYPE_ACTIVE_CONNECTION))
 #define NM_ACTIVE_CONNECTION_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), NM_TYPE_ACTIVE_CONNECTION, NMActiveConnectionClass))
 
-#define NM_ACTIVE_CONNECTION_SERVICE_NAME        "service-name"
 #define NM_ACTIVE_CONNECTION_CONNECTION          "connection"
 #define NM_ACTIVE_CONNECTION_SPECIFIC_OBJECT     "specific-object"
 #define NM_ACTIVE_CONNECTION_DEVICES             "devices"
@@ -67,8 +66,6 @@ GType nm_active_connection_get_type (void);
 
 GObject *nm_active_connection_new (DBusGConnection *connection, const char *path);
 
-const char * nm_active_connection_get_service_name        (NMActiveConnection *connection);
-NMConnectionScope nm_active_connection_get_scope          (NMActiveConnection *connection);
 const char * nm_active_connection_get_connection          (NMActiveConnection *connection);
 const char * nm_active_connection_get_specific_object     (NMActiveConnection *connection);
 const GPtrArray *nm_active_connection_get_devices         (NMActiveConnection *connection);
