@@ -15,7 +15,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * Copyright (C) 2005 - 2008 Red Hat, Inc.
+ * Copyright (C) 2005 - 2011 Red Hat, Inc.
  * Copyright (C) 2007 - 2008 Novell, Inc.
  */
 
@@ -27,7 +27,7 @@
 
 #include "nm-activation-request.h"
 #include "nm-marshal.h"
-#include "nm-utils.h"
+#include "nm-logging.h"
 #include "nm-setting-wireless-security.h"
 #include "nm-setting-8021x.h"
 #include "nm-dbus-manager.h"
@@ -35,21 +35,20 @@
 #include "nm-properties-changed-signal.h"
 #include "nm-active-connection.h"
 #include "nm-dbus-glib-types.h"
+#include "nm-active-connection-glue.h"
+#include "nm-settings-connection.h"
 
-#include "nm-manager.h" /* FIXME! */
 
 G_DEFINE_TYPE (NMActRequest, nm_act_request, G_TYPE_OBJECT)
 
-#define NM_ACT_REQUEST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_TYPE_ACT_REQUEST, NMActRequestPrivate))
+#define NM_ACT_REQUEST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), \
+                                       NM_TYPE_ACT_REQUEST, \
+                                       NMActRequestPrivate))
 
 enum {
-	CONNECTION_SECRETS_UPDATED,
-	CONNECTION_SECRETS_FAILED,
 	PROPERTIES_CHANGED,
-
 	LAST_SIGNAL
 };
-
 static guint signals[LAST_SIGNAL] = { 0 };
 
 typedef struct {
@@ -61,558 +60,133 @@ typedef struct {
 	gboolean disposed;
 
 	NMConnection *connection;
-	DBusGProxyCall *secrets_call;
+
+	GSList *secrets_calls;
 
 	char *specific_object;
 	NMDevice *device;
 	gboolean user_requested;
+	gulong user_uid;
 
 	NMActiveConnectionState state;
 	gboolean is_default;
+	gboolean is_default6;
 	gboolean shared;
 	GSList *share_rules;
 
 	char *ac_path;
+
+	gboolean assumed;
 } NMActRequestPrivate;
 
 enum {
 	PROP_0,
-	PROP_SERVICE_NAME,
 	PROP_CONNECTION,
 	PROP_SPECIFIC_OBJECT,
 	PROP_DEVICES,
 	PROP_STATE,
 	PROP_DEFAULT,
+	PROP_DEFAULT6,
 	PROP_VPN,
 
 	LAST_PROP
 };
 
+/*******************************************************************/
 
-static void
-device_state_changed (NMDevice *device,
-                      NMDeviceState new_state,
-                      NMDeviceState old_state,
-                      NMDeviceStateReason reason,
-                      gpointer user_data)
-{
-	NMActRequest *self = NM_ACT_REQUEST (user_data);
-	NMActRequestPrivate *priv = NM_ACT_REQUEST_GET_PRIVATE (self);
-	NMActiveConnectionState new_ac_state;
-	gboolean new_default = FALSE;
-
-	/* Set NMActiveConnection state based on the device's state */
-	switch (new_state) {
-	case NM_DEVICE_STATE_PREPARE:
-	case NM_DEVICE_STATE_CONFIG:
-	case NM_DEVICE_STATE_NEED_AUTH:
-	case NM_DEVICE_STATE_IP_CONFIG:
-		new_ac_state = NM_ACTIVE_CONNECTION_STATE_ACTIVATING;
-		break;
-	case NM_DEVICE_STATE_ACTIVATED:
-		new_ac_state = NM_ACTIVE_CONNECTION_STATE_ACTIVATED;
-		new_default = priv->is_default;
-		break;
-	default:
-		new_ac_state = NM_ACTIVE_CONNECTION_STATE_UNKNOWN;
-		break;
-	}
-
-	if (new_ac_state != priv->state) {
-		priv->state = new_ac_state;
-		g_object_notify (G_OBJECT (self), NM_ACTIVE_CONNECTION_STATE);
-	}
-
-	if (new_default != priv->is_default) {
-		priv->is_default = new_default;
-		g_object_notify (G_OBJECT (self), NM_ACTIVE_CONNECTION_DEFAULT);
-	}
-}
-
-NMActRequest *
-nm_act_request_new (NMConnection *connection,
-                    const char *specific_object,
-                    gboolean user_requested,
-                    gpointer *device)
-{
-	GObject *object;
-	NMActRequestPrivate *priv;
-
-	g_return_val_if_fail (NM_IS_CONNECTION (connection), NULL);
-	g_return_val_if_fail (NM_DEVICE (device), NULL);
-
-	object = g_object_new (NM_TYPE_ACT_REQUEST, NULL);
-	if (!object)
-		return NULL;
-
-	priv = NM_ACT_REQUEST_GET_PRIVATE (object);
-
-	priv->connection = g_object_ref (connection);
-	if (specific_object)
-		priv->specific_object = g_strdup (specific_object);
-
-	priv->device = NM_DEVICE (device);
-	g_signal_connect (device, "state-changed",
-	                  G_CALLBACK (device_state_changed),
-	                  NM_ACT_REQUEST (object));
-
-	priv->user_requested = user_requested;
-
-	return NM_ACT_REQUEST (object);
-}
-
-static void
-nm_act_request_init (NMActRequest *req)
-{
-	NMActRequestPrivate *priv = NM_ACT_REQUEST_GET_PRIVATE (req);
-	NMDBusManager *dbus_mgr;
-
-	priv->ac_path = nm_active_connection_get_next_object_path ();
-	priv->state = NM_ACTIVE_CONNECTION_STATE_UNKNOWN;
-
-	dbus_mgr = nm_dbus_manager_get ();
-	dbus_g_connection_register_g_object (nm_dbus_manager_get_connection (dbus_mgr),
-	                                     priv->ac_path,
-	                                     G_OBJECT (req));
-	g_object_unref (dbus_mgr);
-}
-
-static void
-cleanup_secrets_dbus_call (NMActRequest *self)
-{
-	NMActRequestPrivate *priv = NM_ACT_REQUEST_GET_PRIVATE (self);
-	DBusGProxy *proxy;
-
-	g_return_if_fail (priv->connection != NULL);
-	g_return_if_fail (NM_IS_CONNECTION (priv->connection));
-
-	proxy = g_object_get_data (G_OBJECT (priv->connection), NM_MANAGER_CONNECTION_SECRETS_PROXY_TAG);
-	g_assert (proxy);
-
-	if (priv->secrets_call) {
-		dbus_g_proxy_cancel_call (proxy, priv->secrets_call);
-		priv->secrets_call = NULL;
-	}
-}
-
-static void
-dispose (GObject *object)
-{
-	NMActRequestPrivate *priv = NM_ACT_REQUEST_GET_PRIVATE (object);
-
-	if (priv->disposed) {
-		G_OBJECT_CLASS (nm_act_request_parent_class)->dispose (object);
-		return;
-	}
-	priv->disposed = TRUE;
-
-	g_assert (priv->connection);
-
-	g_signal_handlers_disconnect_by_func (G_OBJECT (priv->device),
-	                                      G_CALLBACK (device_state_changed),
-	                                      NM_ACT_REQUEST (object));
-
-	cleanup_secrets_dbus_call (NM_ACT_REQUEST (object));
-
-	/* Clear any share rules */
-	nm_act_request_set_shared (NM_ACT_REQUEST (object), FALSE);
-
-	g_object_unref (priv->connection);
-
-	G_OBJECT_CLASS (nm_act_request_parent_class)->dispose (object);
-}
-
-static void
-clear_share_rules (NMActRequest *req)
-{
-	NMActRequestPrivate *priv = NM_ACT_REQUEST_GET_PRIVATE (req);
-	GSList *iter;
-
-	for (iter = priv->share_rules; iter; iter = g_slist_next (iter)) {
-		ShareRule *rule = (ShareRule *) iter->data;
-
-		g_free (rule->table);
-		g_free (rule->rule);
-		g_free (rule);
-	}
-
-	g_slist_free (priv->share_rules);
-	priv->share_rules = NULL;
-}
-
-static void
-finalize (GObject *object)
-{
-	NMActRequestPrivate *priv = NM_ACT_REQUEST_GET_PRIVATE (object);
-
-	g_free (priv->specific_object);
-	g_free (priv->ac_path);
-
-	clear_share_rules (NM_ACT_REQUEST (object));
-
-	G_OBJECT_CLASS (nm_act_request_parent_class)->finalize (object);
-}
-
-static void
-get_property (GObject *object, guint prop_id,
-			  GValue *value, GParamSpec *pspec)
-{
-	NMActRequestPrivate *priv = NM_ACT_REQUEST_GET_PRIVATE (object);
-	GPtrArray *devices;
-
-	switch (prop_id) {
-	case PROP_SERVICE_NAME:
-		nm_active_connection_scope_to_value (priv->connection, value);
-		break;
-	case PROP_CONNECTION:
-		g_value_set_boxed (value, nm_connection_get_path (priv->connection));
-		break;
-	case PROP_SPECIFIC_OBJECT:
-		if (priv->specific_object)
-			g_value_set_boxed (value, priv->specific_object);
-		else
-			g_value_set_boxed (value, "/");
-		break;
-	case PROP_DEVICES:
-		devices = g_ptr_array_sized_new (1);
-		g_ptr_array_add (devices, g_strdup (nm_device_get_udi (priv->device)));
-		g_value_take_boxed (value, devices);
-		break;
-	case PROP_STATE:
-		g_value_set_uint (value, priv->state);
-		break;
-	case PROP_DEFAULT:
-		g_value_set_boolean (value, priv->is_default);
-		break;
-	case PROP_VPN:
-		g_value_set_boolean (value, FALSE);
-		break;
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
-	}
-}
-
-static void
-nm_act_request_class_init (NMActRequestClass *req_class)
-{
-	GObjectClass *object_class = G_OBJECT_CLASS (req_class);
-
-	g_type_class_add_private (req_class, sizeof (NMActRequestPrivate));
-
-	/* virtual methods */
-	object_class->get_property = get_property;
-	object_class->dispose = dispose;
-	object_class->finalize = finalize;
-
-	/* properties */
-	g_object_class_install_property
-		(object_class, PROP_SERVICE_NAME,
-		 g_param_spec_string (NM_ACTIVE_CONNECTION_SERVICE_NAME,
-							  "Service name",
-							  "Service name",
-							  NULL,
-							  G_PARAM_READABLE));
-	g_object_class_install_property
-		(object_class, PROP_CONNECTION,
-		 g_param_spec_boxed (NM_ACTIVE_CONNECTION_CONNECTION,
-							  "Connection",
-							  "Connection",
-							  DBUS_TYPE_G_OBJECT_PATH,
-							  G_PARAM_READABLE));
-	g_object_class_install_property
-		(object_class, PROP_SPECIFIC_OBJECT,
-		 g_param_spec_boxed (NM_ACTIVE_CONNECTION_SPECIFIC_OBJECT,
-							  "Specific object",
-							  "Specific object",
-							  DBUS_TYPE_G_OBJECT_PATH,
-							  G_PARAM_READABLE));
-	g_object_class_install_property
-		(object_class, PROP_DEVICES,
-		 g_param_spec_boxed (NM_ACTIVE_CONNECTION_DEVICES,
-							  "Devices",
-							  "Devices",
-							  DBUS_TYPE_G_ARRAY_OF_OBJECT_PATH,
-							  G_PARAM_READABLE));
-	g_object_class_install_property
-		(object_class, PROP_STATE,
-		 g_param_spec_uint (NM_ACTIVE_CONNECTION_STATE,
-							  "State",
-							  "State",
-							  NM_ACTIVE_CONNECTION_STATE_UNKNOWN,
-							  NM_ACTIVE_CONNECTION_STATE_ACTIVATED,
-							  NM_ACTIVE_CONNECTION_STATE_UNKNOWN,
-							  G_PARAM_READABLE));
-	g_object_class_install_property
-		(object_class, PROP_DEFAULT,
-		 g_param_spec_boolean (NM_ACTIVE_CONNECTION_DEFAULT,
-							   "Default",
-							   "Is the default active connection",
-							   FALSE,
-							   G_PARAM_READABLE));
-	g_object_class_install_property
-		(object_class, PROP_VPN,
-		 g_param_spec_boolean (NM_ACTIVE_CONNECTION_VPN,
-							   "VPN",
-							   "Is a VPN connection",
-							   FALSE,
-							   G_PARAM_READABLE));
-
-	/* Signals */
-	signals[CONNECTION_SECRETS_UPDATED] =
-		g_signal_new ("connection-secrets-updated",
-					  G_OBJECT_CLASS_TYPE (object_class),
-					  G_SIGNAL_RUN_FIRST,
-					  G_STRUCT_OFFSET (NMActRequestClass, connection_secrets_updated),
-					  NULL, NULL,
-					  _nm_marshal_VOID__OBJECT_POINTER_UINT,
-					  G_TYPE_NONE, 3,
-					  G_TYPE_OBJECT, G_TYPE_POINTER, G_TYPE_UINT);
-
-	signals[CONNECTION_SECRETS_FAILED] =
-		g_signal_new ("connection-secrets-failed",
-					  G_OBJECT_CLASS_TYPE (object_class),
-					  G_SIGNAL_RUN_FIRST,
-					  G_STRUCT_OFFSET (NMActRequestClass, connection_secrets_failed),
-					  NULL, NULL,
-					  _nm_marshal_VOID__OBJECT_STRING_UINT,
-					  G_TYPE_NONE, 3,
-					  G_TYPE_OBJECT, G_TYPE_STRING, G_TYPE_UINT);
-
-	signals[PROPERTIES_CHANGED] = 
-		nm_properties_changed_signal_new (object_class,
-								    G_STRUCT_OFFSET (NMActRequestClass, properties_changed));
-
-	nm_active_connection_install_type_info (object_class);
-}
-
-typedef struct GetSecretsInfo {
-	NMActRequest *req;
-	char *setting_name;
-	RequestSecretsCaller caller;
+typedef struct {
+	NMActRequest *self;
+	guint32 call_id;
+	NMActRequestSecretsFunc callback;
+	gpointer callback_data;
 } GetSecretsInfo;
 
 static void
-free_get_secrets_info (gpointer data)
+get_secrets_cb (NMSettingsConnection *connection,
+                guint32 call_id,
+                const char *setting_name,
+                GError *error,
+                gpointer user_data)
 {
-	GetSecretsInfo *info = (GetSecretsInfo *) data;
+	GetSecretsInfo *info = user_data;
+	NMActRequestPrivate *priv = NM_ACT_REQUEST_GET_PRIVATE (info->self);
 
-	g_free (info->setting_name);
+	g_return_if_fail (info->call_id == call_id);
+	priv->secrets_calls = g_slist_remove (priv->secrets_calls, info);
+
+	info->callback (info->self, call_id, NM_CONNECTION (connection), error, info->callback_data);
 	g_free (info);
 }
 
-static void
-update_one_setting (const char* setting_name,
-                    GHashTable *setting_hash,
-                    NMConnection *connection,
-                    GSList **updated)
+guint32
+nm_act_request_get_secrets (NMActRequest *self,
+                            const char *setting_name,
+                            guint32 flags,
+                            const char *hint,
+                            NMActRequestSecretsFunc callback,
+                            gpointer callback_data)
 {
-	GType type;
-	NMSetting *setting = NULL;
-	GError *error = NULL;
+	NMActRequestPrivate *priv;
+	GetSecretsInfo *info;
+	guint32 call_id;
 
-	/* Check whether a complete & valid NMSetting object was returned.  If
-	 * yes, replace the setting object in the connection.  If not, just try
-	 * updating the secrets.
-	 */
-	type = nm_connection_lookup_setting_type (setting_name);
-	if (type == 0)
-		return;
-
-	setting = nm_setting_new_from_hash (type, setting_hash);
-	if (setting) {
-		NMSetting *s_8021x = NULL;
-		GSList *all_settings = NULL;
-
-		/* The wireless-security setting might need the 802.1x setting in
-		 * the all_settings argument of the verify function. Ugh.
-		 */
-		s_8021x = nm_connection_get_setting (connection, NM_TYPE_SETTING_802_1X);
-		if (s_8021x)
-			all_settings = g_slist_append (all_settings, s_8021x);
-
-		if (!nm_setting_verify (setting, all_settings, NULL)) {
-			/* Just try updating secrets */
-			g_object_unref (setting);
-			setting = NULL;
-		}
-
-		g_slist_free (all_settings);
-	}
-
-	if (setting)
-		nm_connection_add_setting (connection, setting);
-	else {
-		if (!nm_connection_update_secrets (connection, setting_name, setting_hash, &error)) {
-			nm_warning ("Failed to update connection secrets: %d %s",
-			            error ? error->code : -1,
-			            error && error->message ? error->message : "(none)");
-			g_clear_error (&error);
-		}
-	}
-
-	*updated = g_slist_append (*updated, (gpointer) setting_name);
-}
-
-static void
-add_one_key_to_list (gpointer key, gpointer data, gpointer user_data)
-{
-	GSList **list = (GSList **) user_data;
-
-	*list = g_slist_append (*list, key);
-}
-
-static gint
-settings_order_func (gconstpointer a, gconstpointer b)
-{
-	/* Just ensure the 802.1x setting gets processed _before_ the
-	 * wireless-security one.
-	 */
-
-	if (   !strcmp (a, NM_SETTING_802_1X_SETTING_NAME)
-	    && !strcmp (b, NM_SETTING_WIRELESS_SECURITY_SETTING_NAME))
-		return -1;
-
-	if (   !strcmp (a, NM_SETTING_WIRELESS_SECURITY_SETTING_NAME)
-	    && !strcmp (b, NM_SETTING_802_1X_SETTING_NAME))
-		return 1;
-
-	return 0;
-}
-
-static void
-get_secrets_cb (DBusGProxy *proxy, DBusGProxyCall *call, gpointer user_data)
-{
-	GetSecretsInfo *info = (GetSecretsInfo *) user_data;
-	GError *err = NULL;
-	GHashTable *settings = NULL;
-	NMActRequestPrivate *priv = NULL;
-	GSList *keys = NULL, *iter;
-	GSList *updated = NULL;
-
-	g_return_if_fail (info != NULL);
-	g_return_if_fail (info->req);
-	g_return_if_fail (info->setting_name);
-
-	priv = NM_ACT_REQUEST_GET_PRIVATE (info->req);
-
-	g_return_if_fail (call == priv->secrets_call);
-	priv->secrets_call = NULL;
-
-	if (!dbus_g_proxy_end_call (proxy, call, &err,
-								DBUS_TYPE_G_MAP_OF_MAP_OF_VARIANT, &settings,
-								G_TYPE_INVALID)) {
-		nm_warning ("Couldn't get connection secrets: %s.", err->message);
-		g_error_free (err);
-		g_signal_emit (info->req,
-		               signals[CONNECTION_SECRETS_FAILED],
-		               0,
-		               priv->connection,
-		               info->setting_name,
-		               info->caller);
-		return;
-	}
-
-	if (g_hash_table_size (settings) == 0) {
-		// FIXME: some better way to handle invalid message?
-		nm_warning ("GetSecrets call returned but no secrets were found.");
-		goto out;
-	}
-
-	g_hash_table_foreach (settings, add_one_key_to_list, &keys);
-	keys = g_slist_sort (keys, settings_order_func);
-	for (iter = keys; iter; iter = g_slist_next (iter)) {
-		GHashTable *setting_hash;
-
-		setting_hash = g_hash_table_lookup (settings, iter->data);
-		if (setting_hash) {
-			update_one_setting ((const char *) iter->data,
-			                    setting_hash,
-			                    priv->connection,
-			                    &updated);
-		} else
-			nm_warning ("Couldn't get setting secrets for '%s'", (const char *) iter->data);
-	}
-	g_slist_free (keys);
-
-	if (g_slist_length (updated)) {
-		g_signal_emit (info->req,
-		               signals[CONNECTION_SECRETS_UPDATED],
-		               0,
-		               priv->connection,
-		               updated,
-		               info->caller);
-	} else {
-		nm_warning ("No secrets updated because not valid settings were received!");
-	}
-
-out:
-	g_slist_free (updated);
-	g_hash_table_destroy (settings);
-}
-
-gboolean
-nm_act_request_request_connection_secrets (NMActRequest *self,
-                                           const char *setting_name,
-                                           gboolean request_new,
-                                           RequestSecretsCaller caller,
-                                           const char *hint1,
-                                           const char *hint2)
-{
-	DBusGProxy *secrets_proxy;
-	GetSecretsInfo *info = NULL;
-	NMActRequestPrivate *priv = NULL;
-	GPtrArray *hints = NULL;
-
-	g_return_val_if_fail (NM_IS_ACT_REQUEST (self), FALSE);
-	g_return_val_if_fail (setting_name != NULL, FALSE);
+	g_return_val_if_fail (self, 0);
+	g_return_val_if_fail (NM_IS_ACT_REQUEST (self), 0);
 
 	priv = NM_ACT_REQUEST_GET_PRIVATE (self);
 
-	cleanup_secrets_dbus_call (self);
-
 	info = g_malloc0 (sizeof (GetSecretsInfo));
-	g_return_val_if_fail (info != NULL, FALSE);
+	info->self = self;
+	info->callback = callback;
+	info->callback_data = callback_data;
 
-	info->req = self;
-	info->caller = caller;
-	info->setting_name = g_strdup (setting_name);
+	call_id = nm_settings_connection_get_secrets (NM_SETTINGS_CONNECTION (priv->connection),
+	                                              priv->user_requested,
+	                                              priv->user_uid,
+	                                              setting_name,
+	                                              flags,
+	                                              hint,
+	                                              get_secrets_cb,
+	                                              info,
+	                                              NULL);
+	if (call_id > 0) {
+		info->call_id = call_id;
+		priv->secrets_calls = g_slist_append (priv->secrets_calls, info);
+	} else
+		g_free (info);
 
-	/* Empty for now */
-	hints = g_ptr_array_sized_new (2);
-
-	if (hint1)
-		g_ptr_array_add (hints, g_strdup (hint1));
-	if (hint2)
-		g_ptr_array_add (hints, g_strdup (hint2));
-
-	secrets_proxy = g_object_get_data (G_OBJECT (priv->connection), NM_MANAGER_CONNECTION_SECRETS_PROXY_TAG);
-	g_assert (secrets_proxy);
-
-	priv->secrets_call = dbus_g_proxy_begin_call_with_timeout (secrets_proxy, "GetSecrets",
-	                                                           get_secrets_cb,
-	                                                           info,
-	                                                           free_get_secrets_info,
-	                                                           G_MAXINT32,
-	                                                           G_TYPE_STRING, setting_name,
-	                                                           DBUS_TYPE_G_ARRAY_OF_STRING, hints,
-	                                                           G_TYPE_BOOLEAN, request_new,
-	                                                           G_TYPE_INVALID);
-	g_ptr_array_free (hints, TRUE);
-	if (!priv->secrets_call) {
-		nm_warning ("Could not call get secrets");
-		goto error;
-	}
-
-	return TRUE;
-
-error:
-	if (info)
-		free_get_secrets_info (info);
-	cleanup_secrets_dbus_call (self);
-	return FALSE;
+	return call_id;
 }
 
+void
+nm_act_request_cancel_secrets (NMActRequest *self, guint32 call_id)
+{
+	NMActRequestPrivate *priv;
+	GSList *iter;
+
+	g_return_if_fail (self);
+	g_return_if_fail (NM_IS_ACT_REQUEST (self));
+	g_return_if_fail (call_id > 0);
+
+	priv = NM_ACT_REQUEST_GET_PRIVATE (self);
+
+	for (iter = priv->secrets_calls; iter; iter = g_slist_next (iter)) {
+		GetSecretsInfo *info = iter->data;
+
+		/* Remove the matching info */
+		if (info->call_id == call_id) {
+			priv->secrets_calls = g_slist_remove_link (priv->secrets_calls, iter);
+			g_slist_free (iter);
+
+			nm_settings_connection_cancel_secrets (NM_SETTINGS_CONNECTION (priv->connection), call_id);
+			g_free (info);
+			break;
+		}
+	}
+}
+
+/*******************************************************************/
+
 NMConnection *
 nm_act_request_get_connection (NMActRequest *req)
 {
@@ -684,6 +258,65 @@ nm_act_request_get_default (NMActRequest *req)
 	return NM_ACT_REQUEST_GET_PRIVATE (req)->is_default;
 }
 
+void
+nm_act_request_set_default6 (NMActRequest *req, gboolean is_default6)
+{
+	NMActRequestPrivate *priv;
+
+	g_return_if_fail (NM_IS_ACT_REQUEST (req));
+
+	priv = NM_ACT_REQUEST_GET_PRIVATE (req);
+	if (priv->is_default6 == is_default6)
+		return;
+
+	priv->is_default6 = is_default6;
+	g_object_notify (G_OBJECT (req), NM_ACTIVE_CONNECTION_DEFAULT6);
+}
+
+gboolean
+nm_act_request_get_default6 (NMActRequest *req)
+{
+	g_return_val_if_fail (NM_IS_ACT_REQUEST (req), FALSE);
+
+	return NM_ACT_REQUEST_GET_PRIVATE (req)->is_default6;
+}
+
+GObject *
+nm_act_request_get_device (NMActRequest *req)
+{
+	g_return_val_if_fail (NM_IS_ACT_REQUEST (req), FALSE);
+
+	return G_OBJECT (NM_ACT_REQUEST_GET_PRIVATE (req)->device);
+}
+
+gboolean
+nm_act_request_get_assumed (NMActRequest *req)
+{
+	g_return_val_if_fail (NM_IS_ACT_REQUEST (req), FALSE);
+
+	return NM_ACT_REQUEST_GET_PRIVATE (req)->assumed;
+}
+
+/********************************************************************/
+
+static void
+clear_share_rules (NMActRequest *req)
+{
+	NMActRequestPrivate *priv = NM_ACT_REQUEST_GET_PRIVATE (req);
+	GSList *iter;
+
+	for (iter = priv->share_rules; iter; iter = g_slist_next (iter)) {
+		ShareRule *rule = (ShareRule *) iter->data;
+
+		g_free (rule->table);
+		g_free (rule->rule);
+		g_free (rule);
+	}
+
+	g_slist_free (priv->share_rules);
+	priv->share_rules = NULL;
+}
+
 static void
 share_child_setup (gpointer user_data G_GNUC_UNUSED)
 {
@@ -713,33 +346,35 @@ nm_act_request_set_shared (NMActRequest *req, gboolean shared)
 		char *envp[1] = { NULL };
 		char **argv;
 		char *cmd;
-		int status;
-		GError *error = NULL;
 
-		if (shared)
-			cmd = g_strdup_printf ("/sbin/iptables --table %s --insert %s", rule->table, rule->rule);
-		else
-			cmd = g_strdup_printf ("/sbin/iptables --table %s --delete %s", rule->table, rule->rule);
+		cmd = g_strdup_printf ("%s --table %s %s %s",
+		                       IPTABLES_PATH,
+		                       rule->table,
+		                       shared ? "--insert" : "--delete",
+		                       rule->rule);
+		if (!cmd)
+			continue;
 
 		argv = g_strsplit (cmd, " ", 0);
-		if (!argv || !argv[0]) {
-			continue;
-			g_free (cmd);
+		if (argv && argv[0]) {
+			int status;
+			GError *error = NULL;
+
+			nm_log_info (LOGD_SHARING, "Executing: %s", cmd);
+			if (!g_spawn_sync ("/", argv, envp, G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
+			                   share_child_setup, NULL, NULL, NULL, &status, &error)) {
+				nm_log_warn (LOGD_SHARING, "Error executing command: (%d) %s",
+				             error ? error->code : -1,
+				             (error && error->message) ? error->message : "(unknown)");
+				g_clear_error (&error);
+			} else if (WEXITSTATUS (status)) {
+				nm_log_warn (LOGD_SHARING, "** Command returned exit status %d.",
+				             WEXITSTATUS (status));
+			}
 		}
-
-		nm_info ("Executing: %s", cmd);
 		g_free (cmd);
-
-		if (!g_spawn_sync ("/", argv, envp, G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
-		                   share_child_setup, NULL, NULL, NULL, &status, &error)) {
-			nm_info ("Error executing command: (%d) %s",
-			         error ? error->code : 0, (error && error->message) ? error->message : "unknown");
-			if (error)
-				g_error_free (error);
-		} else if (WEXITSTATUS (status))
-			nm_info ("** Command returned exit status %d.", WEXITSTATUS (status));
-
-		g_strfreev (argv);
+		if (argv)
+			g_strfreev (argv);
 	}
 
 	g_slist_free (list);
@@ -768,18 +403,230 @@ nm_act_request_add_share_rule (NMActRequest *req,
 	g_return_if_fail (NM_IS_ACT_REQUEST (req));
 	g_return_if_fail (table != NULL);
 	g_return_if_fail (table_rule != NULL);
-	
+
 	rule = g_malloc0 (sizeof (ShareRule));
 	rule->table = g_strdup (table);
 	rule->rule = g_strdup (table_rule);
 	priv->share_rules = g_slist_append (priv->share_rules, rule);
 }
 
-GObject *
-nm_act_request_get_device (NMActRequest *req)
-{
-	g_return_val_if_fail (NM_IS_ACT_REQUEST (req), FALSE);
+/********************************************************************/
 
-	return G_OBJECT (NM_ACT_REQUEST_GET_PRIVATE (req)->device);
+static void
+device_state_changed (NMDevice *device,
+                      NMDeviceState new_state,
+                      NMDeviceState old_state,
+                      NMDeviceStateReason reason,
+                      gpointer user_data)
+{
+	NMActRequest *self = NM_ACT_REQUEST (user_data);
+	NMActRequestPrivate *priv = NM_ACT_REQUEST_GET_PRIVATE (self);
+	NMActiveConnectionState new_ac_state;
+	gboolean new_default = FALSE, new_default6 = FALSE;
+
+	/* Set NMActiveConnection state based on the device's state */
+	switch (new_state) {
+	case NM_DEVICE_STATE_PREPARE:
+	case NM_DEVICE_STATE_CONFIG:
+	case NM_DEVICE_STATE_NEED_AUTH:
+	case NM_DEVICE_STATE_IP_CONFIG:
+		new_ac_state = NM_ACTIVE_CONNECTION_STATE_ACTIVATING;
+		break;
+	case NM_DEVICE_STATE_ACTIVATED:
+		new_ac_state = NM_ACTIVE_CONNECTION_STATE_ACTIVATED;
+		new_default = priv->is_default;
+		new_default6 = priv->is_default6;
+		break;
+	default:
+		new_ac_state = NM_ACTIVE_CONNECTION_STATE_UNKNOWN;
+		break;
+	}
+
+	if (new_ac_state != priv->state) {
+		priv->state = new_ac_state;
+		g_object_notify (G_OBJECT (self), NM_ACTIVE_CONNECTION_STATE);
+	}
+
+	if (new_default != priv->is_default) {
+		priv->is_default = new_default;
+		g_object_notify (G_OBJECT (self), NM_ACTIVE_CONNECTION_DEFAULT);
+	}
+
+	if (new_default6 != priv->is_default6) {
+		priv->is_default6 = new_default6;
+		g_object_notify (G_OBJECT (self), NM_ACTIVE_CONNECTION_DEFAULT6);
+	}
+}
+
+/********************************************************************/
+
+NMActRequest *
+nm_act_request_new (NMConnection *connection,
+                    const char *specific_object,
+                    gboolean user_requested,
+                    gulong user_uid,
+                    gboolean assumed,
+                    gpointer *device)
+{
+	GObject *object;
+	NMActRequestPrivate *priv;
+
+	g_return_val_if_fail (NM_IS_CONNECTION (connection), NULL);
+	g_return_val_if_fail (NM_DEVICE (device), NULL);
+
+	object = g_object_new (NM_TYPE_ACT_REQUEST, NULL);
+	if (!object)
+		return NULL;
+
+	priv = NM_ACT_REQUEST_GET_PRIVATE (object);
+
+	priv->connection = g_object_ref (connection);
+	if (specific_object)
+		priv->specific_object = g_strdup (specific_object);
+
+	priv->device = NM_DEVICE (device);
+	g_signal_connect (device, "state-changed",
+	                  G_CALLBACK (device_state_changed),
+	                  NM_ACT_REQUEST (object));
+
+	priv->user_uid = user_uid;
+	priv->user_requested = user_requested;
+	priv->assumed = assumed;
+
+	return NM_ACT_REQUEST (object);
+}
+
+static void
+nm_act_request_init (NMActRequest *req)
+{
+	NMActRequestPrivate *priv = NM_ACT_REQUEST_GET_PRIVATE (req);
+	NMDBusManager *dbus_mgr;
+
+	priv->ac_path = nm_active_connection_get_next_object_path ();
+	priv->state = NM_ACTIVE_CONNECTION_STATE_UNKNOWN;
+
+	dbus_mgr = nm_dbus_manager_get ();
+	dbus_g_connection_register_g_object (nm_dbus_manager_get_connection (dbus_mgr),
+	                                     priv->ac_path,
+	                                     G_OBJECT (req));
+	g_object_unref (dbus_mgr);
+}
+
+static void
+get_property (GObject *object, guint prop_id,
+			  GValue *value, GParamSpec *pspec)
+{
+	NMActRequestPrivate *priv = NM_ACT_REQUEST_GET_PRIVATE (object);
+	GPtrArray *devices;
+
+	switch (prop_id) {
+	case PROP_CONNECTION:
+		g_value_set_boxed (value, nm_connection_get_path (priv->connection));
+		break;
+	case PROP_SPECIFIC_OBJECT:
+		if (priv->specific_object)
+			g_value_set_boxed (value, priv->specific_object);
+		else
+			g_value_set_boxed (value, "/");
+		break;
+	case PROP_DEVICES:
+		devices = g_ptr_array_sized_new (1);
+		g_ptr_array_add (devices, g_strdup (nm_device_get_path (priv->device)));
+		g_value_take_boxed (value, devices);
+		break;
+	case PROP_STATE:
+		g_value_set_uint (value, priv->state);
+		break;
+	case PROP_DEFAULT:
+		g_value_set_boolean (value, priv->is_default);
+		break;
+	case PROP_DEFAULT6:
+		g_value_set_boolean (value, priv->is_default6);
+		break;
+	case PROP_VPN:
+		g_value_set_boolean (value, FALSE);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+dispose (GObject *object)
+{
+	NMActRequestPrivate *priv = NM_ACT_REQUEST_GET_PRIVATE (object);
+	GSList *iter;
+
+	if (priv->disposed) {
+		G_OBJECT_CLASS (nm_act_request_parent_class)->dispose (object);
+		return;
+	}
+	priv->disposed = TRUE;
+
+	g_signal_handlers_disconnect_by_func (G_OBJECT (priv->device),
+	                                      G_CALLBACK (device_state_changed),
+	                                      NM_ACT_REQUEST (object));
+
+	/* Clear any share rules */
+	nm_act_request_set_shared (NM_ACT_REQUEST (object), FALSE);
+
+	/* Kill any in-progress secrets requests */
+	g_assert (priv->connection);
+	for (iter = priv->secrets_calls; iter; iter = g_slist_next (iter)) {
+		GetSecretsInfo *info = iter->data;
+
+		nm_settings_connection_cancel_secrets (NM_SETTINGS_CONNECTION (priv->connection), info->call_id);
+		g_free (info);
+	}
+	g_slist_free (priv->secrets_calls);
+
+	g_object_unref (priv->connection);
+
+	G_OBJECT_CLASS (nm_act_request_parent_class)->dispose (object);
+}
+
+static void
+finalize (GObject *object)
+{
+	NMActRequestPrivate *priv = NM_ACT_REQUEST_GET_PRIVATE (object);
+
+	g_free (priv->specific_object);
+	g_free (priv->ac_path);
+
+	clear_share_rules (NM_ACT_REQUEST (object));
+
+	G_OBJECT_CLASS (nm_act_request_parent_class)->finalize (object);
+}
+
+static void
+nm_act_request_class_init (NMActRequestClass *req_class)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (req_class);
+
+	g_type_class_add_private (req_class, sizeof (NMActRequestPrivate));
+
+	/* virtual methods */
+	object_class->get_property = get_property;
+	object_class->dispose = dispose;
+	object_class->finalize = finalize;
+
+	/* properties */
+    nm_active_connection_install_properties (object_class,
+                                             PROP_CONNECTION,
+                                             PROP_SPECIFIC_OBJECT,
+                                             PROP_DEVICES,
+                                             PROP_STATE,
+                                             PROP_DEFAULT,
+                                             PROP_DEFAULT6,
+                                             PROP_VPN);
+
+	/* Signals */
+	signals[PROPERTIES_CHANGED] = 
+		nm_properties_changed_signal_new (object_class,
+		                                  G_STRUCT_OFFSET (NMActRequestClass, properties_changed));
+
+	dbus_g_object_type_install_info (G_TYPE_FROM_CLASS (req_class),
+	                                 &dbus_glib_nm_active_connection_object_info);
 }
 
