@@ -16,9 +16,10 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  * Copyright (C) 2008 Novell, Inc.
- * Copyright (C) 2008 Red Hat, Inc.
+ * Copyright (C) 2008 - 2010 Red Hat, Inc.
  */
 
+#include <config.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <signal.h>
@@ -41,6 +42,7 @@
 #include <linux/if_ppp.h>
 
 #include "NetworkManager.h"
+#include "nm-glib-compat.h"
 #include "nm-ppp-manager.h"
 #include "nm-setting-connection.h"
 #include "nm-setting-ppp.h"
@@ -48,7 +50,7 @@
 #include "nm-setting-gsm.h"
 #include "nm-setting-cdma.h"
 #include "nm-dbus-manager.h"
-#include "nm-utils.h"
+#include "nm-logging.h"
 #include "nm-marshal.h"
 
 static void impl_ppp_manager_need_secrets (NMPPPManager *manager,
@@ -65,7 +67,6 @@ static gboolean impl_ppp_manager_set_ip4_config (NMPPPManager *manager,
 #include "nm-ppp-manager-glue.h"
 
 #define NM_PPPD_PLUGIN PLUGINDIR "/nm-pppd-plugin.so"
-#define NM_PPP_WAIT_PPPD 15000 /* 10 seconds */
 #define PPP_MANAGER_SECRET_TRIES "ppp-manager-secret-tries"
 
 typedef struct {
@@ -77,6 +78,7 @@ typedef struct {
 
 	NMActRequest *act_req;
 	DBusGMethodInvocation *pending_secrets_context;
+	guint32 secrets_id;
 
 	guint32 ppp_watch_id;
 	guint32 ppp_timeout_handler;
@@ -135,12 +137,11 @@ constructor (GType type,
 	GObject *object;
 	NMPPPManagerPrivate *priv;
 	DBusGConnection *connection;
-	static gboolean name_requested = FALSE;
 	static guint32 counter = 0;
 
 	object = G_OBJECT_CLASS (nm_ppp_manager_parent_class)->constructor (type,
-														   n_construct_params,
-														   construct_params);
+	                                                                    n_construct_params,
+	                                                                    construct_params);
 	if (!object)
 		return NULL;
 
@@ -150,36 +151,8 @@ constructor (GType type,
 		g_object_unref (object);
 		return NULL;
 	}
+
 	connection = nm_dbus_manager_get_connection (priv->dbus_manager);
-
-	/* Only need to request bus name the first time */
-	if (!name_requested) {
-		DBusGProxy *proxy;
-		gboolean success;
-		guint request_name_result;
-		GError *err = NULL;
-
-		proxy = dbus_g_proxy_new_for_name (connection,
-									"org.freedesktop.DBus",
-									"/org/freedesktop/DBus",
-									"org.freedesktop.DBus");
-		success = dbus_g_proxy_call (proxy, "RequestName", &err,
-		                             G_TYPE_STRING, NM_DBUS_SERVICE_PPP,
-		                             G_TYPE_UINT, 0,
-		                             G_TYPE_INVALID,
-		                             G_TYPE_UINT, &request_name_result,
-		                             G_TYPE_INVALID);
-		g_object_unref (proxy);
-
-		if (!success) {
-			nm_warning ("Failed to acquire PPP manager service: %s", err->message);
-			g_object_unref (object);
-			return NULL;
-		}
-
-		name_requested = TRUE;
-	}
-
 	priv->dbus_path = g_strdup_printf (NM_DBUS_PATH "/PPP/%d", counter++);
 	dbus_g_connection_register_g_object (connection, priv->dbus_path, object);
 
@@ -193,7 +166,8 @@ dispose (GObject *object)
 
 	nm_ppp_manager_stop (NM_PPP_MANAGER (object));
 
-	g_object_unref (priv->act_req);
+	if (priv->act_req)
+		g_object_unref (priv->act_req);
 	g_object_unref (priv->dbus_manager);
 
 	G_OBJECT_CLASS (nm_ppp_manager_parent_class)->dispose (object);
@@ -325,12 +299,13 @@ monitor_cb (gpointer user_data)
 	req.stats_ptr = (caddr_t) &req.stats;
 
 	strncpy (req.ifr__name, priv->ip_iface, sizeof (req.ifr__name));
-	if (ioctl (priv->monitor_fd, SIOCGPPPSTATS, &req) < 0)
-		nm_warning ("Could not read ppp stats: %s", strerror (errno));
-	else
+	if (ioctl (priv->monitor_fd, SIOCGPPPSTATS, &req) < 0) {
+		nm_log_warn (LOGD_PPP, "could not read ppp stats: %s", strerror (errno));
+	} else {
 		g_signal_emit (manager, signals[STATS], 0, 
-					req.stats.p.ppp_ibytes,
-					req.stats.p.ppp_obytes);
+		               req.stats.p.ppp_ibytes,
+		               req.stats.p.ppp_obytes);
+	}
 
 	return TRUE;
 }
@@ -342,9 +317,9 @@ monitor_stats (NMPPPManager *manager)
 
 	priv->monitor_fd = socket (AF_INET, SOCK_DGRAM, 0);
 	if (priv->monitor_fd > 0)
-		priv->monitor_id = g_timeout_add (5000, monitor_cb, manager);
+		priv->monitor_id = g_timeout_add_seconds (5, monitor_cb, manager);
 	else
-		nm_warning ("Could not open pppd monitor: %s", strerror (errno));
+		nm_log_warn (LOGD_PPP, "could not monitor PPP stats: %s", strerror (errno));
 }
 
 /*******************************************/
@@ -361,19 +336,29 @@ remove_timeout_handler (NMPPPManager *manager)
 }
 
 static void
-impl_ppp_manager_need_secrets (NMPPPManager *manager,
-                               DBusGMethodInvocation *context)
+cancel_get_secrets (NMPPPManager *self)
 {
-	NMPPPManagerPrivate *priv = NM_PPP_MANAGER_GET_PRIVATE (manager);
-	NMConnection *connection;
-	NMSettingConnection *s_con;
-	const char *connection_type;
-	const char *setting_name;
-	guint32 tries;
-	GPtrArray *hints = NULL;
-	const char *hint1 = NULL, *hint2 = NULL;
+	NMPPPManagerPrivate *priv = NM_PPP_MANAGER_GET_PRIVATE (self);
 
-	connection = nm_act_request_get_connection (priv->act_req);
+	if (priv->secrets_id) {
+		nm_act_request_cancel_secrets (priv->act_req, priv->secrets_id);
+		priv->secrets_id = 0;
+	}
+}
+
+static gboolean
+extract_details_from_connection (NMConnection *connection,
+                                 const char **username,
+                                 const char **password,
+                                 GError **error)
+{
+	NMSettingConnection *s_con;
+	NMSetting *setting;
+	const char *connection_type;
+
+	g_return_val_if_fail (connection != NULL, FALSE);
+	g_return_val_if_fail (username != NULL, FALSE);
+	g_return_val_if_fail (password != NULL, FALSE);
 
 	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
 	g_assert (s_con);
@@ -381,67 +366,117 @@ impl_ppp_manager_need_secrets (NMPPPManager *manager,
 	connection_type = nm_setting_connection_get_connection_type (s_con);
 	g_assert (connection_type);
 
+	setting = nm_connection_get_setting_by_name (connection, connection_type);
+	if (!setting) {
+		g_set_error_literal (error, NM_PPP_MANAGER_ERROR, NM_PPP_MANAGER_ERROR_UNKOWN,
+		                     "Missing type-specific setting; no secrets could be found.");
+		return FALSE;
+	}
+
+	/* FIXME: push this down to the settings and keep PPP manager generic */
+	if (NM_IS_SETTING_PPPOE (setting)) {
+		*username = nm_setting_pppoe_get_username (NM_SETTING_PPPOE (setting));
+		*password = nm_setting_pppoe_get_password (NM_SETTING_PPPOE (setting));
+	} else if (NM_IS_SETTING_GSM (setting)) {
+		*username = nm_setting_gsm_get_username (NM_SETTING_GSM (setting));
+		*password = nm_setting_gsm_get_password (NM_SETTING_GSM (setting));
+	} else if (NM_IS_SETTING_CDMA (setting)) {
+		*username = nm_setting_cdma_get_username (NM_SETTING_CDMA (setting));
+		*password = nm_setting_cdma_get_password (NM_SETTING_CDMA (setting));
+	}
+
+	return TRUE;
+}
+
+static void
+ppp_secrets_cb (NMActRequest *req,
+                guint32 call_id,
+                NMConnection *connection,
+                GError *error,
+                gpointer user_data)
+{
+	NMPPPManager *self = NM_PPP_MANAGER (user_data);
+	NMPPPManagerPrivate *priv = NM_PPP_MANAGER_GET_PRIVATE (self);
+	const char *username = NULL;
+	const char *password = NULL;
+	GError *local = NULL;
+
+	g_return_if_fail (priv->pending_secrets_context != NULL);
+	g_return_if_fail (req == priv->act_req);
+	g_return_if_fail (call_id == priv->secrets_id);
+
+	if (error) {
+		nm_log_warn (LOGD_PPP, "%s", error->message);
+		dbus_g_method_return_error (priv->pending_secrets_context, error);
+		goto out;
+	}
+
+	if (!extract_details_from_connection (connection, &username, &password, &local)) {
+		nm_log_warn (LOGD_PPP, "%s", local->message);
+		dbus_g_method_return_error (priv->pending_secrets_context, local);
+		g_clear_error (&local);
+		goto out;
+	}
+
+	/* This is sort of a hack but...
+	 * pppd plugin only ever needs username and password. Passing the full
+	 * connection there would mean some bloat: the plugin would need to link
+	 * against libnm-util just to parse this. So instead, let's just send what
+	 * it needs.
+	 */
+	dbus_g_method_return (priv->pending_secrets_context, username, password);
+
+out:
+	priv->pending_secrets_context = NULL;
+	priv->secrets_id = 0;
+}
+
+static void
+impl_ppp_manager_need_secrets (NMPPPManager *manager,
+                               DBusGMethodInvocation *context)
+{
+	NMPPPManagerPrivate *priv = NM_PPP_MANAGER_GET_PRIVATE (manager);
+	NMConnection *connection;
+	const char *setting_name;
+	const char *username = NULL;
+	const char *password = NULL;
+	guint32 tries;
+	GPtrArray *hints = NULL;
+	GError *error = NULL;
+	guint32 flags = NM_ACT_REQUEST_GET_SECRETS_FLAG_ALLOW_INTERACTION;
+
+	connection = nm_act_request_get_connection (priv->act_req);
+
 	nm_connection_clear_secrets (connection);
 	setting_name = nm_connection_need_secrets (connection, &hints);
 	if (!setting_name) {
-		NMSetting *setting;
-
-		setting = nm_connection_get_setting_by_name (connection, connection_type);
-		if (setting) {
-			const char *username = NULL;
-			const char *password = NULL;
-
-			/* FIXME: push this down to the settings and keep PPP manager generic */
-			if (NM_IS_SETTING_PPPOE (setting)) {
-				username = nm_setting_pppoe_get_username (NM_SETTING_PPPOE (setting));
-				password = nm_setting_pppoe_get_password (NM_SETTING_PPPOE (setting));
-			} else if (NM_IS_SETTING_GSM (setting)) {
-				username = nm_setting_gsm_get_username (NM_SETTING_GSM (setting));
-				password = nm_setting_gsm_get_password (NM_SETTING_GSM (setting));
-			} else if (NM_IS_SETTING_CDMA (setting)) {
-				username = nm_setting_cdma_get_username (NM_SETTING_CDMA (setting));
-				password = nm_setting_cdma_get_password (NM_SETTING_CDMA (setting));
-			}
-
-			/* If secrets are not required, send the existing username and password
-			 * back to the PPP plugin immediately.
-			 */
+		/* Use existing secrets from the connection */
+		if (extract_details_from_connection (connection, &username, &password, &error)) {
+			/* Send existing secrets to the PPP plugin */
 			priv->pending_secrets_context = context;
-			nm_ppp_manager_update_secrets (manager,
-			                               priv->parent_iface,
-			                               username ? username : "",
-			                               password ? password : "",
-			                               NULL);
+			ppp_secrets_cb (priv->act_req, priv->secrets_id, connection, NULL, manager);
 		} else {
-			GError *err = NULL;
-
-			g_set_error (&err, NM_PPP_MANAGER_ERROR, NM_PPP_MANAGER_ERROR_UNKOWN,
-					   "Missing type-specific setting; no secrets could be found.");
-			nm_warning ("%s", err->message);
-			dbus_g_method_return_error (context, err);
+			nm_log_warn (LOGD_PPP, "%s", error->message);
+			dbus_g_method_return_error (priv->pending_secrets_context, error);
+			g_clear_error (&error);
 		}
 		return;
 	}
 
-	/* Extract hints */
-	if (hints) {
-		if (hints->len > 0)
-			hint1 = g_ptr_array_index (hints, 0);
-		if (hints->len > 1)
-			hint2 = g_ptr_array_index (hints, 1);
-	}
-
-	tries = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (connection), PPP_MANAGER_SECRET_TRIES));
-	/* Only ask for completely new secrets after retrying them once; some PPP
-	 * servers (T-Mobile USA) appear to ask a few times when they actually don't
-	 * even care what you pass back.
+	/* Only ask for completely new secrets after retrying them once; some devices
+	 * appear to ask a few times when they actually don't even care what you
+	 * pass back.
 	 */
-	nm_act_request_request_connection_secrets (priv->act_req,
-	                                           setting_name,
-	                                           tries > 1 ? TRUE : FALSE,
-	                                           SECRETS_CALLER_PPP,
-	                                           hint1,
-	                                           hint2);
+	tries = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (connection), PPP_MANAGER_SECRET_TRIES));
+	if (tries > 1)
+		flags |= NM_ACT_REQUEST_GET_SECRETS_FLAG_REQUEST_NEW;
+
+	priv->secrets_id = nm_act_request_get_secrets (priv->act_req,
+	                                               setting_name,
+	                                               flags,
+	                                               hints ? g_ptr_array_index (hints, 0) : NULL,
+	                                               ppp_secrets_cb,
+	                                               manager);
 	g_object_set_data (G_OBJECT (connection), PPP_MANAGER_SECRET_TRIES, GUINT_TO_POINTER (++tries));
 	priv->pending_secrets_context = context;
 
@@ -465,12 +500,13 @@ impl_ppp_manager_set_ip4_config (NMPPPManager *manager,
 {
 	NMPPPManagerPrivate *priv = NM_PPP_MANAGER_GET_PRIVATE (manager);
 	NMConnection *connection;
+	NMSettingPPP *s_ppp;
 	NMIP4Config *config;
 	NMIP4Address *addr;
 	GValue *val;
 	int i;
 
-	nm_info ("PPP manager(IP Config Get) reply received.");
+	nm_log_info (LOGD_PPP, "PPP manager(IP Config Get) reply received.");
 
 	remove_timeout_handler (manager);
 
@@ -495,7 +531,7 @@ impl_ppp_manager_set_ip4_config (NMPPPManager *manager,
 	if (nm_ip4_address_get_address (addr) && nm_ip4_address_get_prefix (addr)) {
 		nm_ip4_config_take_address (config, addr);
 	} else {
-		nm_warning ("%s: invalid IPv4 address received!", __func__);
+		nm_log_err (LOGD_PPP, "invalid IPv4 address received!");
 		nm_ip4_address_unref (addr);
 		goto out;
 	}
@@ -508,9 +544,17 @@ impl_ppp_manager_set_ip4_config (NMPPPManager *manager,
 			nm_ip4_config_add_nameserver (config, g_array_index (dns, guint, i));
 	}
 
+	val = (GValue *) g_hash_table_lookup (config_hash, NM_PPP_IP4_CONFIG_WINS);
+	if (val) {
+		GArray *wins = (GArray *) g_value_get_boxed (val);
+
+		for (i = 0; i < wins->len; i++)
+			nm_ip4_config_add_wins (config, g_array_index (wins, guint, i));
+	}
+
 	val = (GValue *) g_hash_table_lookup (config_hash, NM_PPP_IP4_CONFIG_INTERFACE);
 	if (!val || !G_VALUE_HOLDS_STRING (val)) {
-		nm_warning ("No interface");
+		nm_log_err (LOGD_PPP, "no interface received!");
 		goto out;
 	}
 	priv->ip_iface = g_value_dup_string (val);
@@ -520,6 +564,15 @@ impl_ppp_manager_set_ip4_config (NMPPPManager *manager,
 	g_assert (connection);
 	g_object_set_data (G_OBJECT (connection), PPP_MANAGER_SECRET_TRIES, NULL);
 
+	/* Merge in custom MTU */
+	s_ppp = (NMSettingPPP *) nm_connection_get_setting (connection, NM_TYPE_SETTING_PPP);
+	if (s_ppp) {
+		guint32 mtu = nm_setting_ppp_get_mtu (s_ppp);
+
+		if (mtu)
+			nm_ip4_config_set_mtu (config, mtu);
+	}
+
 	/* Push the IP4 config up to the device */
 	g_signal_emit (manager, signals[IP4_CONFIG], 0, priv->ip_iface, config);
 
@@ -593,13 +646,12 @@ nm_cmd_line_add_int (NMCmdLine *cmd, int i)
 static inline const char *
 nm_find_pppd (void)
 {
-	static const char *pppd_binary_paths[] =
-		{
-			"/usr/local/sbin/pppd",
-			"/usr/sbin/pppd",
-			"/sbin/pppd",
-			NULL
-		};
+	static const char *pppd_binary_paths[] = {
+		"/usr/local/sbin/pppd",
+		"/usr/sbin/pppd",
+		"/sbin/pppd",
+		NULL
+	};
 
 	const char  **pppd_binary = pppd_binary_paths;
 
@@ -681,7 +733,7 @@ ppp_exit_code (guint pppd_exit_status, GPid pid)
 		msg = "Unknown error";
 	}
 
-	nm_warning ("ppp pid %d exited with error: %s", pid, msg);
+	nm_log_warn (LOGD_PPP, "pppd pid %d exited with error: %s", pid, msg);
 }
 
 static void
@@ -697,14 +749,14 @@ ppp_watch_cb (GPid pid, gint status, gpointer user_data)
 		err = WEXITSTATUS (status);
 		if (err != 0)
 			ppp_exit_code (err, priv->pid);
-	} else if (WIFSTOPPED (status))
-		nm_warning ("ppp pid %d stopped unexpectedly with signal %d", priv->pid, WSTOPSIG (status));
-	else if (WIFSIGNALED (status))
-		nm_warning ("ppp pid %d died with signal %d", priv->pid, WTERMSIG (status));
-	else
-		nm_warning ("ppp pid %d died from an unknown cause", priv->pid);
+	} else if (WIFSTOPPED (status)) {
+		nm_log_info (LOGD_PPP, "pppd pid %d stopped unexpectedly with signal %d", priv->pid, WSTOPSIG (status));
+	} else if (WIFSIGNALED (status)) {
+		nm_log_info (LOGD_PPP, "pppd pid %d died with signal %d", priv->pid, WTERMSIG (status));
+	} else
+		nm_log_info (LOGD_PPP, "pppd pid %d died from an unknown cause", priv->pid);
 
-	nm_debug ("ppp pid %d cleaned up", priv->pid);
+	nm_log_dbg (LOGD_PPP, "pppd pid %d cleaned up", priv->pid);
 	priv->pid = 0;
 	g_signal_emit (manager, signals[STATE_CHANGED], 0, NM_PPP_STATUS_DEAD);
 }
@@ -714,7 +766,7 @@ pppd_timed_out (gpointer data)
 {
 	NMPPPManager *manager = NM_PPP_MANAGER (data);
 
-	nm_warning ("Looks like pppd didn't initialize our dbus module");
+	nm_log_warn (LOGD_PPP, "pppd timed out or didn't initialize our dbus module");
 	nm_ppp_manager_stop (manager);
 
 	g_signal_emit (manager, signals[STATE_CHANGED], 0, NM_PPP_STATUS_DEAD);
@@ -734,6 +786,8 @@ create_pppd_cmd_line (NMPPPManager *self,
 	NMCmdLine *cmd;
 	const char *ppp_debug;
 
+	g_return_val_if_fail (setting != NULL, NULL);
+
 	ppp_binary = nm_find_pppd ();
 	if (!ppp_binary) {
 		g_set_error (err, NM_PPP_MANAGER_ERROR, NM_PPP_MANAGER_ERROR,
@@ -785,8 +839,12 @@ create_pppd_cmd_line (NMPPPManager *self,
 	if (nm_setting_ppp_get_baud (setting))
 		nm_cmd_line_add_int (cmd, nm_setting_ppp_get_baud (setting));
 
-	if (nm_setting_ppp_get_noauth (setting))
-		nm_cmd_line_add_string (cmd, "noauth");
+	/* noauth by default, because we certainly don't have any information
+	 * with which to verify anything the peer gives us if we ask it to
+	 * authenticate itself, which is what 'auth' really means.
+	 */
+	nm_cmd_line_add_string (cmd, "noauth");
+
 	if (nm_setting_ppp_get_refuse_eap (setting))
 		nm_cmd_line_add_string (cmd, "refuse-eap");
 	if (nm_setting_ppp_get_refuse_pap (setting))
@@ -885,11 +943,13 @@ gboolean
 nm_ppp_manager_start (NMPPPManager *manager,
                       NMActRequest *req,
                       const char *ppp_name,
+                      guint32 timeout_secs,
                       GError **err)
 {
 	NMPPPManagerPrivate *priv;
 	NMConnection *connection;
-	NMSettingPPP *ppp_setting;
+	NMSettingPPP *s_ppp;
+	gboolean s_ppp_created = FALSE;
 	NMSettingPPPOE *pppoe_setting;
 	NMCmdLine *ppp_cmd;
 	char *cmd_str;
@@ -899,30 +959,49 @@ nm_ppp_manager_start (NMPPPManager *manager,
 	g_return_val_if_fail (NM_IS_PPP_MANAGER (manager), FALSE);
 	g_return_val_if_fail (NM_IS_ACT_REQUEST (req), FALSE);
 
+	priv = NM_PPP_MANAGER_GET_PRIVATE (manager);
+
+#if !WITH_PPP
+	/* PPP support disabled */
+	g_set_error_literal (err,
+	                     NM_PPP_MANAGER_ERROR,
+	                     NM_PPP_MANAGER_ERROR_UNKOWN,
+	                     "PPP support is not enabled.");
+	return FALSE;
+#endif
+
+	priv->pid = 0;
+
 	/* Make sure /dev/ppp exists (bgo #533064) */
 	if (stat ("/dev/ppp", &st) || !S_ISCHR (st.st_mode))
 		ignored = system ("/sbin/modprobe ppp_generic");
 
 	connection = nm_act_request_get_connection (req);
-	ppp_setting = NM_SETTING_PPP (nm_connection_get_setting (connection, NM_TYPE_SETTING_PPP));
-	g_return_val_if_fail (ppp_setting != NULL, FALSE);
+	g_assert (connection);
+
+	s_ppp = (NMSettingPPP *) nm_connection_get_setting (connection, NM_TYPE_SETTING_PPP);
+	if (!s_ppp) {
+		/* If the PPP settings are all default we may not have a PPP setting yet,
+		 * so just make a default one here.
+		 */
+		s_ppp = NM_SETTING_PPP (nm_setting_ppp_new ());
+		s_ppp_created = TRUE;
+	}
 	
 	pppoe_setting = (NMSettingPPPOE *) nm_connection_get_setting (connection, NM_TYPE_SETTING_PPPOE);
 	if (pppoe_setting)
-		pppoe_fill_defaults (ppp_setting);
+		pppoe_fill_defaults (s_ppp);
 
-	ppp_cmd = create_pppd_cmd_line (manager, ppp_setting, pppoe_setting, ppp_name, err);
+	ppp_cmd = create_pppd_cmd_line (manager, s_ppp, pppoe_setting, ppp_name, err);
 	if (!ppp_cmd)
-		return FALSE;
+		goto out;
 
 	g_ptr_array_add (ppp_cmd->array, NULL);
 
-	priv = NM_PPP_MANAGER_GET_PRIVATE (manager);
-
-	nm_info ("Starting pppd connection");
+	nm_log_info (LOGD_PPP, "starting PPP connection");
 
 	cmd_str = nm_cmd_line_to_str (ppp_cmd);
-	nm_debug ("Command line: %s", cmd_str);
+	nm_log_dbg (LOGD_PPP, "command line: %s", cmd_str);
 	g_free (cmd_str);
 
 	priv->pid = 0;
@@ -933,59 +1012,22 @@ nm_ppp_manager_start (NMPPPManager *manager,
 		goto out;
 	}
 
-	nm_debug ("ppp started with pid %d", priv->pid);
+	nm_log_info (LOGD_PPP, "pppd started with pid %d", priv->pid);
 
 	priv->ppp_watch_id = g_child_watch_add (priv->pid, (GChildWatchFunc) ppp_watch_cb, manager);
-	priv->ppp_timeout_handler = g_timeout_add (NM_PPP_WAIT_PPPD, pppd_timed_out, manager);
+	priv->ppp_timeout_handler = g_timeout_add_seconds (timeout_secs, pppd_timed_out, manager);
 	priv->act_req = g_object_ref (req);
 
  out:
+	if (s_ppp_created)
+		g_object_unref (s_ppp);
+
 	if (ppp_cmd)
 		nm_cmd_line_destroy (ppp_cmd);
 
 	return priv->pid > 0;
 }
 
-void
-nm_ppp_manager_update_secrets (NMPPPManager *manager,
-                               const char *device,
-                               const char *username,
-                               const char *password,
-                               const char *error_message)
-{
-	NMPPPManagerPrivate *priv = NM_PPP_MANAGER_GET_PRIVATE (manager);
-
-	g_return_if_fail (NM_IS_PPP_MANAGER (manager));
-	g_return_if_fail (device != NULL);
-	g_return_if_fail (priv->pending_secrets_context != NULL);
-
-	if (error_message) {
-		g_return_if_fail (username == NULL);
-		g_return_if_fail (password == NULL);
-	} else {
-		g_return_if_fail (username != NULL);
-		g_return_if_fail (password != NULL);
-	}
-
-	if (error_message) {
-		GError *err = NULL;
-
-		g_set_error (&err, NM_PPP_MANAGER_ERROR, NM_PPP_MANAGER_ERROR_UNKOWN, "%s", error_message);
-		nm_warning ("%s", error_message);
-		dbus_g_method_return_error (priv->pending_secrets_context, err);
-		g_error_free (err);
-	} else {
-		/* This is sort of a hack but...
-		   pppd plugin only ever needs username and password.
-		   Passing the full connection there would mean some bloat:
-		   the plugin would need to link against libnm-util just to parse this.
-		   So instead, let's just send what it needs */
-
-		dbus_g_method_return (priv->pending_secrets_context, username, password);
-	}
-	priv->pending_secrets_context = NULL;
-}
-
 static gboolean
 ensure_killed (gpointer data)
 {
@@ -995,9 +1037,9 @@ ensure_killed (gpointer data)
 		kill (pid, SIGKILL);
 
 	/* ensure the child is reaped */
-	nm_debug ("waiting for ppp pid %d to exit", pid);
+	nm_log_dbg (LOGD_PPP, "waiting for pppd pid %d to exit", pid);
 	waitpid (pid, NULL, 0);
-	nm_debug ("ppp pid %d cleaned up", pid);
+	nm_log_dbg (LOGD_PPP, "pppd pid %d cleaned up", pid);
 
 	return FALSE;
 }
@@ -1011,6 +1053,8 @@ nm_ppp_manager_stop (NMPPPManager *manager)
 
 	priv = NM_PPP_MANAGER_GET_PRIVATE (manager);
 
+	cancel_get_secrets (manager);
+
 	if (priv->monitor_id) {
 		g_source_remove (priv->monitor_id);
 		priv->monitor_id = 0;
@@ -1035,14 +1079,14 @@ nm_ppp_manager_stop (NMPPPManager *manager)
 
 	if (priv->pid) {
 		if (kill (priv->pid, SIGTERM) == 0)
-			g_timeout_add (2000, ensure_killed, GINT_TO_POINTER (priv->pid));
+			g_timeout_add_seconds (2, ensure_killed, GINT_TO_POINTER (priv->pid));
 		else {
 			kill (priv->pid, SIGKILL);
 
 			/* ensure the child is reaped */
-			nm_debug ("waiting for ppp pid %d to exit", priv->pid);
+			nm_log_dbg (LOGD_PPP, "waiting for pppd pid %d to exit", priv->pid);
 			waitpid (priv->pid, NULL, 0);
-			nm_debug ("ppp pid %d cleaned up", priv->pid);
+			nm_log_dbg (LOGD_PPP, "pppd pid %d cleaned up", priv->pid);
 		}
 
 		priv->pid = 0;
