@@ -985,7 +985,6 @@ write_wired_setting (NMConnection *connection, shvarFile *ifcfg, GError **error)
 static void
 write_connection_setting (NMSettingConnection *s_con, shvarFile *ifcfg)
 {
-	char *tmp;
 	guint32 n, i;
 	GString *str;
 
@@ -995,13 +994,6 @@ write_connection_setting (NMSettingConnection *s_con, shvarFile *ifcfg)
 	            nm_setting_connection_get_autoconnect (s_con) ? "yes" : "no",
 	            FALSE);
 
-	svSetValue (ifcfg, "LAST_CONNECT", NULL, FALSE);
-	if (nm_setting_connection_get_timestamp (s_con)) {
-		tmp = g_strdup_printf ("%" G_GUINT64_FORMAT, nm_setting_connection_get_timestamp (s_con));
-		svSetValue (ifcfg, "LAST_CONNECT", tmp, FALSE);
-		g_free (tmp);
-	}
-
 	/* Permissions */
 	svSetValue (ifcfg, "USERS", NULL, FALSE);
 	n = nm_setting_connection_get_num_permissions (s_con);
